/*
 *  overlay.js
 *  httProxy oevrlay event handling
 *
 */

// constants
const GT_URL = "http://www.google.com/translate?langpair=en|en&u=";
const ANON_STYLE_URL = "A";
const CDN_STYLE_URL = "C";
const PKBLOG_STYLE_URL = "P";
const PROXY_SEP = "##~##";
const DET_SEP = "#~#";

// globals
var proxyNames = new Array(20)
var proxyURLs = new Array(20)
var proxyStyles = new Array(20)
var numProxies = 0
var proxyMenuListModified = true;
var proxyListModified = true;
var newProxyList = "";

// httProxy logic
var httproxy = {

  menuItems: new Array("httproxy-view", "httproxy-viewNewTab"),
  linkMenuItems: new Array("httproxy-viewlink", "httproxy-viewlinkNewTab"),
  
  readOptions: function()
  {
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    var proxies = null;
    var proxyList = null;
    var proxyDetails = null;
    var defaultProxy = this.getDefaultProxy();
  
    if (!proxyListModified)
      return;
    
    proxies = prefs.getCharPref("extensions.httproxy.proxies");
    
    // separate proxies
    proxyList = proxies ? proxies.split(PROXY_SEP) : "";
    
    // separate proxy names and other info
    for (var i = 0; i < proxyList.length && i < 20; i++) {
       if (proxyList[i] != '') {
         proxyDetails = proxyList[i] ? proxyList[i].split(DET_SEP) : "";
         proxyNames[i] = proxyDetails[0];
         proxyURLs[i] = proxyDetails[1];
         proxyStyles[i] = proxyDetails[2];
       }
    }
    numProxies = i;
    proxyMenuListModified = true;
    
    //Components.utils.reportError("Read Options; numProxies: " + numProxies); 
  },
  
  getDefaultProxy: function()
  {
    var defaultProxy = -1;
  
    var prefservice = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
    var prefs = prefservice.getBranch("");
    if (prefs.getPrefType("extensions.httproxy.defaultproxy") == prefs.PREF_INT)
      defaultProxy = prefs.getIntPref("extensions.httproxy.defaultproxy");
      
    return defaultProxy;
  },
  
  startsWith: function(st, pref)
  {
    return( (pref.length > 0) && (st.substring(0, pref.length) == pref) );
      
  },
  
  checkUrl: function(url)
  {
    return ( typeof(url) != 'undefined'
      && url.substr
      && this.startsWith(url, 'http://') );
  },
  
  proxifyURL: function(url, proxyIndex) 
  {
   if (proxyIndex == null)
     proxyIndex = this.getDefaultProxy();
     
   if (this.checkUrl(url))
   {
     // convert to proxy url
     var proxyURL = proxyURLs[proxyIndex];
     var newUrl = null;
     
     // is this a CDN style proxy
     if (proxyStyles[proxyIndex] == ANON_STYLE_URL) {
       newUrl = proxyURLs[proxyIndex] + url;
     }
     else if (proxyStyles[proxyIndex] == CDN_STYLE_URL) {
        var myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/]+)(:\[0-9]+)?(/(.*))?");
        var regExRes = myRegEx.exec(url);
        if (regExRes == null || regExRes[1] != "http:")
            return "error";
            
        newUrl = "http://" + regExRes[2]
        // append CDN proxy name as required
        if (regExRes[3] != null)   // there is a port part
            newUrl = newUrl + "." + (new String(regExRes[3]).substr(1));
        newUrl = newUrl + proxyURLs[proxyIndex] + ((regExRes[5] == null) ? "" : regExRes[5]);
     }
     else if (proxyStyles[proxyIndex] == PKBLOG_STYLE_URL) {
        var myRegEx = new RegExp("^([^:]+:)?/{2,}([^:/.]+)(\.blogspot\.com)(:\[0-9]+)?(/.*)?");
        var regExRes = myRegEx.exec(url);
        if (regExRes == null || regExRes[1] != "http:")
            return "error";
        newUrl = proxyURLs[proxyIndex] + regExRes[2] + ((regExRes[5] == null) ? "" : regExRes[5])
     }
     else
       return "error";
     
     return newUrl;
   }
   else
     return "error";
  },
  
  openURL: function(url, newTab)
  {
   if (newTab)
     gBrowser.addTab(url);
   else
     window._content.location = url;
  },
  
  proxyViewURL: function() 
  {
       var url = document.getElementById('urlbar').value;
       url = this.proxifyURL (url);
       if (url != "error")
         this.openURL(url, false);
       else
         alert ("Invalid URL to be proxied.");
  },
  
  proxyViewURLUsingProxy: function(oMenuItem) 
  {
       var index = oMenuItem.getAttribute("index");
       // idnetify and use proxy server properly
       if (index < 0 || index > numProxies)
         return false;
       
       var url = document.getElementById('urlbar').value;
       url = this.proxifyURL (url, index);
       if (url != "error") {
         this.openURL(url, false);
         return true;
       }
       else {
         alert ("Invalid URL to be proxied.");
         return false;
       }
  },
  
  proxyViewPage: function(newTab) 
  {
       var url = gBrowser.currentURI.spec;
       url = this.proxifyURL (url);
       if (url != "error")
         this.openURL(url, newTab);
       else
         alert ("Invalid URL to be proxied.");
  },
  
  proxyViewLink: function(newTab) 
  {
    if (gContextMenu)
    {
       var url = this.contextLinkURL();
       url = this.proxifyURL (url);
       if (url != "error")
         this.openURL(url, newTab);
       else
         alert ("Invalid URL to be proxied.");
    }
  },
  
  contextLinkURL: function()
  {
  if (gContextMenu && gContextMenu.getLinkURL)
  {
      return(gContextMenu.getLinkURL());
  }
  
  return(false);
  },
  
  addNewProxy: function()
  {
   var proxyList = document.getElementById('proxylist');
   var newName = document.getElementById('newName').value;
   var style = document.getElementById('style').selectedItem.value;
   var newUrl = document.getElementById('newUrl').value;
   var error = false;
   
   if (style == "")
     error = true;
   else if (style != CDN_STYLE_URL && !this.startsWith(newUrl, 'http://'))
     error = true;
   else if (newName == "")
     error = true;
   
   if (error) {
     alert ("Invalid Proxy Data");
     return;
   }
   
   if (proxyList.childNodes.length >= 20) {
     alert ("Sorry, only 20 proxies can be added. This is a limitation with current version :(");
     return;
   }
   
   newProxyList = newProxyList + PROXY_SEP + newName + DET_SEP + newUrl + DET_SEP + style;
   proxyList.appendItem(newName + " - " + newUrl);
   proxyListModified = true;
  },
  
  readSettingsOptions: function()
  {
    var proxyList = document.getElementById('proxylist');
    this.readOptions();
    
    for (var i = 0; i < numProxies; i++) {
      proxyList.appendItem(proxyNames[i] + " - " + proxyURLs[i]);
    }
    
    proxyList.clearSelection();
    proxyList.selectedIndex = this.getDefaultProxy();
  },
  
  setSettingsOptions: function()
  {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefService);
    var prefs = prefService.getBranch("");
    var proxyList = document.getElementById('proxylist');
    var selected = proxyList.selectedIndex;
    var proxies = prefs.getCharPref("extensions.httproxy.proxies");
  
    if (selected < 0)
    {
      alert ("Invalid Selection");
      return;
    }
  
    // Option to add user-defined proxies
    prefs.setCharPref("extensions.httproxy.proxies", proxies + newProxyList);
    prefs.setIntPref("extensions.httproxy.defaultproxy", selected);
    // after change in settings, redraw the settings menu
    proxyListModified = true;
    
    // update proxy options - for any change in proxies
    this.readOptions();
    window.close();
  },

  setDefaultProxy: function (oMenuItem)
  {
    var index = oMenuItem.getAttribute("index");
    // idnetify and use proxy server properly
    if (index < 0 || index > numProxies)
      return false;

    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                         .getService(Components.interfaces.nsIPrefService);
    var prefs = prefService.getBranch("");
    prefs.setIntPref("extensions.httproxy.defaultproxy", index);
    // after change in settings, redraw the settings menu
    proxyMenuListModified = true;
    
    return true;
  },
   
  clearProxyList: function(aParent)
  {
	if(aParent!=null){
		var aProxyListElem = aParent.getElementsByAttribute("class", "proxy-name");
		for(var c = aProxyListElem.length-1; c >= 0 ; c--){
          if (aProxyListElem[c].getAttribute("parent") == aParent.id) {
            aParent.removeChild(aProxyListElem[c]);
          }
		}
	}
    return true;
  },
   
  fillProxyList: function(aParent)
  {
    var oItem = null;
    var command;

    this.readOptions();

    //Components.utils.reportError("fillProxyList Options; numProxies: " + numProxies 
      //+ " proxyMenuListModified: " + proxyMenuListModified + " parent: " + aParent.id); 

    if (aParent.id == "httpproxy-popup-default-proxy")
      command = " httproxy.setDefaultProxy(this); event.preventBubble();";
    else if (aParent.id == "httproxy-toolbar-menu")
      command = " httproxy.proxyViewURLUsingProxy(this);  event.preventBubble();";
   
    // Remove old entries if any
    this.clearProxyList(aParent);
    
    for (var i = 0; i < numProxies; i++) {
      // create menu item
      oItem = document.createElement("menuitem");
      oItem.setAttribute("id", aParent.id + "-proxy-" + i);
      oItem.setAttribute("parent", aParent.id);
      oItem.setAttribute("label", proxyNames[i]);
      oItem.setAttribute("tooltiptext", proxyURLs[i]);
      oItem.setAttribute("type", "checkbox");
      oItem.setAttribute("index", i);
      oItem.setAttribute("proxyStyle", proxyStyles[i]);
      oItem.setAttribute("class", "proxy-name");
      oItem.setAttribute("oncommand", command);

      if (i == this.getDefaultProxy()){
        //createMenuItem(aParent, i, "** " + proxyNames[i]);
        oItem.setAttribute("checked", true);
      }
      else {
        //createMenuItem(aParent, i, proxyNames[i]);
        oItem.setAttribute("checked", false);
      }
      // append this menu item to the menu
      aParent.appendChild(oItem);
    }
    
    proxyMenuListModified = false;
    return true;
  },
   

  httproxyContext: function(aEvent)
  {
    if (gContextMenu)
    {
       var i;
       var menuitem = null;
       
       var anyMenuItemHidden =
         gContextMenu.isTextSelected || // neglect selected text
         gContextMenu.onImage ||        // neglect images
         gContextMenu.onTextInput;      // neglect text input boxes
       var menuItemHidden = 
         gContextMenu.onLink ||         // neglect links
         anyMenuItemHidden ||           
         !this.checkUrl(gBrowser.currentURI.spec); // check URL
       var linkMenuItemHidden =
         !gContextMenu.onLink ||        // only show if link
         anyMenuItemHidden ||
         !this.checkUrl(this.contextLinkURL());   // check Link URL
       
       //Components.utils.reportError("anyMenuItemHidden: " + anyMenuItemHidden);
       //Components.utils.reportError("menuItemHidden: " + menuItemHidden);
       //Components.utils.reportError("linkMenuItemHidden: " + linkMenuItemHidden);
       
       for ( i=0; i < this.menuItems.length; i++)
       {
          menuitem = document.getElementById(this.menuItems[i]);
          if (menuitem)
             menuitem.hidden = menuItemHidden;
       }
       for ( i=0; i < this.linkMenuItems.length; i++)
       {
          menuitem = document.getElementById(this.linkMenuItems[i]);
          if (menuitem)
             menuitem.hidden = linkMenuItemHidden;
       }
    }
  },
  
  onLoad: function() {
    //Components.utils.reportError("httproxy.onload: " + this);
  
    var menu = document.getElementById("contentAreaContextMenu");
    if (menu)
    {
       menu.addEventListener("popupshowing", httproxyContextListener,false);
    }
    proxyListModified = true;
    // read proxy options
    this.readOptions();
    // initialization code
    this.initialized = true;
    this.httProxyInitialShow();
    return true;
  },
   
  // show custom buttons only in the initial startup
  httProxyInitialShow: function() {
      var bar = document.getElementById('nav-bar');
      if (bar && bar.currentSet) {
          var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                             .getService(Components.interfaces.nsIPrefService);
          var prefs = prefService.getBranch("");
          
          var currentset = bar.currentSet;
          var buttons = currentset.replace(/__empty/, '').split(',');
  
          if (!prefs.getBoolPref('httproxy.initialshow.buttons')) {
              if (currentset.indexOf('httproxy-button') < 0) {
                  if (currentset.indexOf('go-container') < 0) {
                      buttons.push('httproxy-button');
                  }
                  else {
                      currentset = currentset.replace(/go-container/, 'go-container,httproxy-button');
                      buttons = currentset.split(',');
                  }
              }
              prefs.setBoolPref('httproxy.initialshow.buttons', true);
          }
          currentset = bar.currentSet.replace(/__empty/, '');
          var newset = buttons.join(',');
          if (currentset != newset) {
              bar.currentSet = newset;
              bar.setAttribute('currentset', newset);
              document.persist(bar.id, 'currentset');
          }
      }
  }
};

function httproxyContextListener(e)
{
  return(httproxy.httproxyContext(e));
}

var gHttpProxyInitialized = false;

function httproxyInitListener(e)
{
  if (gHttpProxyInitialized) return false;
  gHttpProxyInitialized = true;
  return(httproxy.onLoad(e));
}

function readSettingsOptions()
{
  return (httproxy.readSettingsOptions());
}

function displayHttproxyOptions()
{
    window.open("chrome://httproxy/content/httproxySettings.xul", "", "chrome");
}

window.addEventListener("load", httproxyInitListener, false);

